@@ -2,11 +2,13 @@ import subprocess
 import json
 import logging
 from abc import ABCMeta, abstractmethod
+from os import environ as env
 from ._py_op_items import (
     OPItemFactory,
     OPAbstractItem,
     OPLoginItem,
 )
+from ._py_op_cli import OPCLIConfig
 from ._py_op_deprecation import deprecated
 
 
@@ -33,8 +35,10 @@ class OPSigninException(_OPAbstractException):
 class OPLookupException(_OPAbstractException):
     MSG = "1Password lookup failed."
 
-    def __init__(self, stderr_out, returncode):
-        super().__init__(stderr_out, returncode, self.MSG)
+    def __init__(self, stderr_out, returncode, msg=None):
+        if msg is None:
+            msg = self.MSG
+        super().__init__(stderr_out, returncode, msg)
 
 
 # For now have this class extend OPLookupException
@@ -76,7 +80,7 @@ class OP:
     Class for logging into and querying a 1Password account via the 'op' cli command.
     """
 
-    def __init__(self, signin_address=None, email_address=None,
+    def __init__(self, account_shorthand=None, signin_address=None, email_address=None,
                  secret_key=None, password=None, logger=None, op_path='op'):
         """
         Create an OP object. The 1Password sign-in happens during object instantiation.
@@ -86,32 +90,58 @@ class OP:
         otherwise, a normal sign-in is performed. See `op --help` for further explanation.
 
         Arguments:
-            - 'op_path': optional path to the `op` command, if it's not at the default location
+            - 'account_shorthand': The shorthand name for the account on this device.
+                                   You may choose this during initial signin, otherwise
+                                   1Password converts it from your account address.
+                                   See 'op signin --help' for more information.
             - 'signin_address': Fully qualified address of the 1Password account.
                                 E.g., 'my-account.1password.com'
             - 'email_address': Email of the address for the user of the account
             - 'secret_key': Secret key for the account
             - 'password': The user's master password
             - 'logger': A logging object. If not provided a basic logger is created and used.
+            - 'op_path': optional path to the `op` command, if it's not at the default location
 
         Raises:
             - OPSigninException if 1Password sign-in fails for any reason.
             - OPNotFoundException if the 1Password command can't be found.
         """
-        self.op_path = op_path
         if not logger:
             logging.basicConfig(format="%(message)s", level=logging.DEBUG)
             logger = logging.getLogger()
-
         self.logger = logger
-        initial_signin_args = [signin_address,
-                               email_address, secret_key, password]
+
+        if account_shorthand is None:
+            config = OPCLIConfig()
+            try:
+                account_shorthand = config['latest_signin']
+                self.logger.debug(
+                    "Using account shorthand found in op config: {}".format(account_shorthand))
+            except KeyError:
+                account_shorthand = None
+
+        if account_shorthand is None:
+            raise OPSigninException(
+                "Account shorthand not provided and not found in 'op' config")
+
+        self.account_shorthand = account_shorthand
+        self.op_path = op_path
+
+        initial_signin_args = [account_shorthand,
+                               signin_address,
+                               email_address,
+                               secret_key,
+                               password]
         initial_signin = (None not in initial_signin_args)
 
         if initial_signin:
             self.token = self._do_initial_signin(*initial_signin_args)
+            # export OP_SESSION_<signin_address>
         else:
             self.token = self._do_normal_signin(password)
+        sess_var_name = 'OP_SESSION_{}'.format(self.account_shorthand)
+        # TODO: return alread-decoded token from sign-in
+        env[sess_var_name] = self.token.decode()
 
     def _do_normal_signin(self, password):
         self.logger.info("Doing normal (non-initial) 1Password sign-in")
@@ -120,7 +150,7 @@ class OP:
         token = self._run_signin(signin_argv, password=password).rstrip()
         return token
 
-    def _do_initial_signin(self, signin_address, email_address, secret_key, password):
+    def _do_initial_signin(self, account_shorthand, signin_address, email_address, secret_key, password):
         self.logger.info(
             "Performing initial 1Password sign-in to {} as {}".format(signin_address, email_address))
         signin_argv = [self.op_path, "signin", signin_address,
@@ -133,10 +163,10 @@ class OP:
     def _run_signin(self, argv, password=None):
         return self._run(argv, OPSigninException, capture_stdout=True, input_string=password)
 
-    def _run_get_item(self, argv, input_string, decode=None):
-        return self._run(argv, OPLookupException, capture_stdout=True, input_string=input_string, decode=decode)
+    def _run_get_item(self, argv, input_string=None, decode=None):
+        return self._run(argv, OPGetItemException, capture_stdout=True, input_string=input_string, decode=decode)
 
-    def _run_get_document(self, argv, input_string, decode=None):
+    def _run_get_document(self, argv, input_string=None, decode=None):
         return self._run(argv, OPGetDocumentException, capture_stdout=True, input_string=input_string, decode=decode)
 
     def _run(self, argv, op_exception_class, capture_stdout=False, input_string=None, decode=None):
@@ -147,7 +177,7 @@ class OP:
                 input_string = input_string.encode("utf-8")
         try:
             _ran = subprocess.run(argv, input=input_string,
-                                  stderr=subprocess.PIPE, stdout=stdout)
+                                  stderr=subprocess.PIPE, stdout=stdout, env=env)
         except FileNotFoundError as err:
             self.logger.error(
                 "1Password 'op' command not found at: {}".format(argv[0]))
@@ -171,7 +201,7 @@ class OP:
 
     def get_item(self, item_name_or_uuid):
         lookup_argv = [self.op_path, "get", "item", item_name_or_uuid]
-        output = self._run_get_item(lookup_argv, self.token, decode="utf-8")
+        output = self._run_get_item(lookup_argv, decode="utf-8")
         item_dict = json.loads(output)
         op_item = OPItemFactory.op_item_from_item_dict(item_dict)
         return op_item
@@ -222,7 +252,7 @@ class OP:
                 "Item has no 'fileName' attribute") from ae
         get_document_argv = [self.op_path,
                              "get", "document", document_name_or_uuid]
-        document_bytes = self._run_get_document(get_document_argv, self.token)
+        document_bytes = self._run_get_document(get_document_argv)
 
         return (file_name, document_bytes)
 
@@ -273,6 +303,6 @@ class OP:
             - Bytes of the specified document
         """
         lookup_argv = [self.op_path, "get", "document", item_name_or_uuid]
-        output = self._run_get_document(lookup_argv, self.token)
+        output = self._run_get_document(lookup_argv)
 
         return output
